<?
require_once("conf/dokuwiki.php");
require_once("inc/io.php");

//set up error reporting to sane values
error_reporting(E_ALL ^ E_NOTICE);

//kill magic quotes
if (get_magic_quotes_gpc()) {
  if (!empty($_GET))    remove_magic_quotes($_GET);
  if (!empty($_POST))   remove_magic_quotes($_POST);
  if (!empty($_COOKIE)) remove_magic_quotes($_COOKIE);
  if (!empty($_REQUEST)) remove_magic_quotes($_REQUEST);
  if (!empty($_SESSION)) remove_magic_quotes($_SESSION);
  ini_set('magic_quotes_gpc', 0);
}
set_magic_quotes_runtime(0);

function remove_magic_quotes(&$array) {
  foreach (array_keys($array) as $key) {
    if (is_array($array[$key])) {
      remove_magic_quotes($array[$key]);
    }else {
      $array[$key] = stripslashes($array[$key]);
    }
  } 
} 

//disable gzip if not available
if($conf['usegzip'] && !function_exists('gzopen')){
  $conf['usegzip'] = 0;
}

/* ---------------------------------------------------------------------------------- */

/**
 * This returns the full absolute URL to the directory where
 * DokuWiki is installed in (includes a trailing slash)
 */
function getBaseURL($abs=false){
  global $conf;
  //if canonical url enabled always return absolute
  if($conf['canonical']) $abs = true;

  //relative URLs are easy
  if(!$abs){
    $dir = dirname($_SERVER['PHP_SELF']).'/';
    $dir = preg_replace('#//#','/',$dir);
    return $dir;
  }

  $port = ':'.$_SERVER['SERVER_PORT'];
  //remove port from hostheader as sent by IE
  $host = preg_replace('/:.*$/','',$_SERVER['HTTP_HOST']);

  // see if HTTPS is enabled - apache leaves this empty when not available,
  // IIS sets it to 'off', 'false' and 'disabled' are just guessing
  if (preg_match('/^(|off|false|disabled)$/i',$_SERVER['HTTPS'])){
    $proto = 'http://';
    if ($_SERVER['SERVER_PORT'] == '80') {
      $port='';
    }
  }else{
    $proto = 'https://';
    if ($_SERVER['SERVER_PORT'] == '443') {
      $port='';
    }
  }
  $dir = (dirname($_SERVER['PHP_SELF'])).'/';
  $dir = preg_replace('#//#','/',$dir);

  return $proto.$host.$port.$dir;
}

/*
 * Returns info about the current document as associative
 * array.
 */
function pageinfo(){
  global $ID;
  global $REV;

  $info['namespace'] = getNS($ID);
  $info['locked']    = checklock($ID);
  $info['filepath']  = realpath(wikiFN($ID,$REV));
  $info['exists']    = file_exists($info['filepath']);
  $info['writable']  = (!$info['exists'] || is_writable($info['filepath']));
  $info['editable']  = ($info['writable'] && empty($info['lock']));
  $info['lastmod']   = @filemtime($info['filepath']);
  
  return $info;
}

/**
 * This builds the breadcrumbstrail and returns it as array
 */
function breadcrumbs(){
  global $ID;
  global $ACT;
  global $conf;
  $crumbs = $_SESSION[$conf['title']]['bc'];
  
  //first visit?
  if (!is_array($crumbs)){
    $crumbs = array();
  }
  //we only save on show and existing wiki documents
  if($ACT != 'show' || !file_exists(wikiFN($ID))){
    $_SESSION[$conf['title']]['bc'] = $crumbs;
    return $crumbs;
  }
  //remove ID from array
  $pos = array_search($ID,$crumbs);
  if($pos !== false && $pos !== null){
    array_splice($crumbs,$pos,1);
  }

  //add to array
  $crumbs[] =$ID;
  //reduce size
  while(count($crumbs) > $conf['breadcrumbs']){
    array_shift($crumbs);
  }
  //save to session
  $_SESSION[$conf['title']]['bc'] = $crumbs;
  return $crumbs;
}


/**
 * This builds a link to a wikipage (using getBaseURL)
 */
function wl($id='',$more='',$script='doku.php',$canonical=false){
  global $conf;
  $more = str_replace(',','&amp;',$more);

  $xlink = getBaseURL($canonical);

  if(!$conf['userewrite']){
    $xlink .= $script;
    $xlink .= '?id='.$id;
    if($more) $xlink .= '&amp;'.$more;
  }else{
    $xlink .= $id;
    if($more) $xlink .= '?'.$more;
  }

  return $xlink;
}

/**
 * Just builds a link to a script
 */
function script($script='doku.php'){
  $link = getBaseURL();
  $link .= $script;
  return $link;
}

/**
 * Return namespacepart of a wiki ID
 */
function getNS($id){
 if(strpos($id,':')!==false){
   return substr($id,0,strrpos($id,':'));
 }
 return false;
}

/**
 * Checks the wikitext against a list of blocked expressions
 * returns true if the text contains any bad words
 */
function checkwordblock(){
  global $TEXT;
  global $conf;

  if(!$conf['usewordblock']) return false;

  $blocks = file('conf/wordblock.conf');
  $re = array();
  #build regexp from blocks
  foreach($blocks as $block){
    $block = preg_replace('/#.*$/','',$block);
    $block = trim($block);
    if(empty($block)) continue;
    $re[]  = $block;
  }
  if(preg_match('#('.join('|',$re).')#si',$TEXT)) return true;
  return false;
}

/**
 * Returns the IP of the client including X-Forwarded-For
 * Proxy Headers
 */
function clientIP(){
  $my = $_SERVER['REMOTE_ADDR'];
  if($_SERVER['HTTP_X_FORWARDED_FOR']){
    $my .= ' ('.$_SERVER['HTTP_X_FORWARDED_FOR'].')';
  }
  return $my;
}

/**
 * Checks if a given page is currently locked by anyone for editing.
 * removes stale lockfiles
 */
function checklock($id){
  global $conf;
  $lock = wikiFN($id).'.lock';
  
  //no lockfile
  if(!file_exists($lock)) return false;
  
  //lockfile expired
  if((time() - filemtime($lock)) > $conf['locktime']){
    unlink($lock);
    return false;
  }
  
  //my own lock
  $ip = io_readFile($lock);
  if($ip == clientIP()){
    return false;
  }
  
  return $ip;
}

/**
 * Locks a page for editing
 */
function lock($id){
  $lock = wikiFN($id).'.lock';
  io_saveFile($lock,clientIP());
}

/**
 * Unlocks a page if it was locked by the user
 */
function unlock($id){
  $lock = wikiFN($id).'.lock';
  if(file_exists($lock)){
    $ip = io_readFile($lock);
    if($ip == clientIP()){
      @unlink($lock);
    }
  }
}

/**
 * Cleans a given ID to only use allowed characters. Accented characters are
 * converted to unaccented ones
 */
function cleanID($id){
  $id = trim($id);
  $id = strtolower($id);

  // replace accented chars with unaccented ones
  // this may look strange on your terminal - just don't touch
  $id = strtr(
  strtr($id,
   '',
   'szszyaaaaaaceeeeiiiinoooooouuuuyaaaaaceeeeiiiinooooouuuyy'),
  array('' => 'th', '' => 'th', '' => 'dh', '' => 'dh', '' => 'ss',
   '' => 'oe', '' => 'oe', '' => 'ae', '' => 'ae', '' => 'u',
   '' => 'ue', '' => 'oe', '' => 'ae'));

  //special chars left will be converted to _
  $id = preg_replace('#[^a-z0-9:\-\.]#','_',$id);
  $id = preg_replace('#__#','_',$id);
  $id = preg_replace('#:+#',':',$id);
  $id = trim($id,':._-');
  $id = preg_replace('#:[:\._\-]+#',':',$id);
  return($id);
}

/**
 * returns the full path to the datafile specified by ID and
 * optional revision
 */
function wikiFN($id,$rev=''){
  global $conf;
  $id = cleanID($id);
  $id = str_replace(':','/',$id);
  if(empty($rev)){
    return $conf['datadir'].'/'.$id.'.txt';
  }else{
    $fn = $conf['olddir'].'/'.$id.'.'.$rev.'.txt';
    if(!$conf['usegzip'] || file_exists($fn)){
      //return plaintext if exists or gzip is disabled
      return $fn;
    }else{
      return $fn.'.gz';
    }
  }
}

/**
 * Returns the full filepath to a localized textfile if local
 * version isn't found the english one is returned
 */
function localeFN($id){
  global $conf;
  $file = './lang/'.$conf['lang'].'/'.$id.'.txt';
  if(!file_exists($file)){
    //fall back to english
    $file = './lang/en/'.$id.'.txt';
  }
  return cleanText($file);
}

/**
 * convert line ending to unix format
 *
 * @see: formText() for 2crlf conversion
 */
function cleanText($text){
  $text = preg_replace("/(\015\012)|(\015)/","\012",$text);
  return $text;
}

/**
 * Prepares text for print in Webforms by encoding special chars.
 * It also converts line endings to Windows format which is
 * pseudo standard for webforms. 
 *
 * @see: cleanText() for 2unix conversion
 */
function formText($text){
  $text = preg_replace("/\012/","\015\012",$text);
  return htmlspecialchars($text);
}

/**
 * Returns the specified textfile in parsed format
 */
function parsedLocale($id){
  //disable section editing
  global $parser;
  $se = $parser['secedit'];
  $parser['secedit'] = false;
  //fetch parsed locale
  $html = io_cacheParse(localeFN($id));
  //reset section editing
  $parser['secedit'] = $se;
  return $html;
}

/**
 * Returns the specified textfile in parsed format
 */
function rawLocale($id){
  return io_readFile(localeFN($id));
}


/**
 * Returns the parsed Wikitext for the given id and revision. If $excuse
 * is true an explanation is returned if the file wasn't found
 */
function parsedWiki($id,$rev='',$excuse=true){
  $file = wikiFN($id,$rev);
  $ret  = '';
  
  //ensure $id is in global $ID (needed for parsing)
  global $ID;
  $ID = $id;
  
  if($rev){
    if(file_exists($file)){
      $ret = parse(io_readFile($file));
    }elseif($excuse){
      $ret = parsedLocale('norev');
    }
  }else{
    if(file_exists($file)){
      $ret = io_cacheParse($file);
    }elseif($excuse){
      $ret = parsedLocale('newpage');
    }
  }
  return $ret;
}

/**
 * Returns the raw WikiText
 */
function rawWiki($id,$rev=''){
  return io_readFile(wikiFN($id,$rev));
}

/**
 * Returns the raw Wiki Text in three slices. The range parameter
 * Need to have the form "from-to" and gives the range of the section.
 * The returned order is prefix, section and suffix.
 */
function rawWikiSlices($range,$id,$rev=''){
  list($from,$to) = split('-',$range,2);
  $text = io_readFile(wikiFN($id,$rev));
  $text = split("\n",$text);
  if(!$from) $from = 0;
  if(!$to)   $to   = count($text);

  $slices[0] = join("\n",array_slice($text,0,$from));
  $slices[1] = join("\n",array_slice($text,$from,$to + 1  - $from));
  $slices[2] = join("\n",array_slice($text,$to+1));

  return $slices;
}

/**
 * function to join the text slices with correct lineendings again.
 * When the pretty parameter is set to true it adds additional empty
 * lines between sections if needed (used on saving).
 */
function con($pre,$text,$suf,$pretty=false){

  if($pretty){
    if($pre && substr($pre,-1) != "\n") $pre .= "\n";
    if($suf && substr($text,-1) != "\n") $text .= "\n";
  }

  if($pre) $pre .= "\n";
  if($suf) $text .= "\n";
  return $pre.$text.$suf;
}

/**
 * little function to print the content of a var
 */
function dbg($msg,$hidden=false){
  (!$hidden) ? print '<pre class="dbg">' : print "<!--\n";
  print_r($msg);
  (!$hidden) ? print '</pre>' : print "\n-->";
}

/**
 * Add's an entry to the changelog
 */
function addLogEntry($id){
  global $conf;
  $id     = cleanID($id);
  $date   = time();
  $remote = $_SERVER['REMOTE_ADDR'];
  $user   = $_SERVER['REMOTE_USER'];

  $logline = join("\t",array($date,$remote,$id,$user))."\n";

  $fh = fopen($conf['changelog'],'a');
  if($fh){
    fwrite($fh,$logline);
    fclose($fh);
  }
}

/**
 * returns an array of recently changed files using the
 * changelog
 */
function getRecents($num=0){
  global $conf;
  $recent = array();
  if(!$num) $num = $conf['recent'];

  $loglines = file($conf['changelog']);
  rsort($loglines); //reverse sort on timestamp

  foreach ($loglines as $line){
    $line = rtrim($line);        //remove newline
    $info = split("\t",$line);   //split into parts
    //add id if not in yet and file still exists
    if(!$recent[$info[2]] && file_exists(wikiFN($info[2]))){
      $recent[$info[2]]['date'] = $info[0];
      $recent[$info[2]]['ip']   = $info[1];
      $recent[$info[2]]['user'] = $info[3];
    }
    if(count($recent) >= $num){
      break; //finish if enough items found
    }
  }
  return $recent;
}

/**
 * Saves a wikitext by calling io_saveFile
 */
function saveWikiText($id,$text){
  global $conf;
  umask($conf['umask']);
  // ignore if no changes were made
  if($text == rawWiki($id,'')){
    return;
  }

  $file = wikiFN($id);
  $old = saveOldRevision($id);
  io_saveFile($file,$text);
  addLogEntry($id);
  notify($id,$old);
  
  //purge cache on add by updating the purgefile
  if($conf['purgeonadd'] && !$old){
    io_saveFile($conf['datadir'].'/.cache/purgefile',time());
  }
}

function saveOldRevision($id){
	global $conf;
  umask($conf['umask']);
  $oldf = wikiFN($id);
  if(!file_exists($oldf)) return '';
  $date = filemtime($oldf);
  $newf = wikiFN($id,$date);
  if(substr($newf,-3)=='.gz'){
    io_saveFile($newf,rawWiki($id));
  }else{
    io_makeFileDir($newf);
    copy($oldf, $newf);
  }
  return $date;
}

function notify($id,$rev=""){
  global $lang;
  global $conf;
  if(empty($conf['notify'])) return; //notify enabled?
  
  $text = rawLocale('mailtext');
  $text = str_replace('@DATE@',date($conf['dformat']),$text);
  $text = str_replace('@BROWSER@',$_SERVER['HTTP_USER_AGENT'],$text);
  $text = str_replace('@IPADDRESS@',$_SERVER['REMOTE_ADDR'],$text);
  $text = str_replace('@HOSTNAME@',gethostbyaddr($_SERVER['REMOTE_ADDR']),$text);
  $text = str_replace('@NEWPAGE@',wl($id,'','',true),$text);
  $text = str_replace('@DOKUWIKIURL@',getBaseURL(true),$text);
  
  if($rev){
    $subject = $lang['mail_changed'].' '.$id;
    $text = str_replace('@OLDPAGE@',wl($id,"rev=$rev",'',true),$text);
    require_once("inc/DifferenceEngine.php");
    $df  = new Diff(split("\n",rawWiki($id,$rev)),
                    split("\n",rawWiki($id)));
    $dformat = new UnifiedDiffFormatter();
    $diff    = $dformat->format($df);
  }else{
    $subject=$lang['mail_newpage'].' '.$id;
    $text = str_replace('@OLDPAGE@','none',$text);
    $diff = rawWiki($id);
  }
  $text = str_replace('@DIFF@',$diff,$text);

  @mail($conf['notify'],$subject,$text);
}

function getRevisions($id){
  $revd = dirname(wikiFN($id,'foo'));
  $revs = array();
  $clid = cleanID($id);
  if(strrpos($clid,':')) $clid = substr($clid,strrpos($clid,':')+1); //remove path

  if (is_dir($revd) && $dh = opendir($revd)) {
    while (($file = readdir($dh)) !== false) {
      if (is_dir($revd.'/'.$file)) continue;
      if (preg_match('/^'.$clid.'\.(\d+)\.txt(\.gz)?$/',$file,$match)){
        $revs[]=$match[1];
      }
    }
    closedir($dh);
  }
  rsort($revs);
  return $revs;
}

/*
function quicksearch($q){
  global $conf;
  $hits = array();
  $cmd = $conf['find']." ".$conf['datadir']."/ -type f -iname '*$q*'";
  $results = io_runcmd($cmd);
  $results = split("\n",$results);
  foreach($results as $result){
    preg_match('#^'.preg_quote($conf['datadir'],'#').'/(.+)(\.txt)$#s',$result,$matches);
    $matches[1] = trim($matches[1]);
    if(empty($matches[1])) continue;
    $hits[] = str_replace('/',':',$matches[1]);
  }
  return $hits;
}

function fulltextsearch($q){
  global $conf;
  $cmd = $conf['grep']." -i -c -r -- '$q' ".$conf['datadir']." |".$conf['sort']." -t':' -k 2 -n -r";
  $results = io_runcmd($cmd);
  $results = split("\n",$results);
  foreach($results as $result){
    preg_match('#^'.preg_quote($conf['datadir'],'#').'/(.+)(\.txt:)(\d+)#s',$result,$matches);
    if($matches[3] == 0) continue;
    $matches[1] = str_replace('/',':',$matches[1]);
    $cmd  = $conf['grep']." -i -C 1 -- '$q' ".wikiFN($matches[1]);
    $snip = io_runcmd($cmd);
    $snip = substr($snip,0,1000);
    html_result($matches[1],$matches[3],$q,$snip);
  }
}

function recursedir($base,$rec='',$runf='',$rund='',$ext='',$dir='',$lvl=1){
  $dirs   = array();
  $data   = array();
  $files  = array();

  $dh = opendir($base.'/'.$dir);
  if(!$dh) return $data;
  while(($file = readdir($dh)) !== false){
    if(preg_match('/^\./',$file)) continue; //skip hidden files and upper dirs
    if(is_dir($base.'/'.$dir.'/'.$file)){
      $dirs[] = $dir.'/'.$file;
      continue;
    }
    if(!preg_match('#'.$ext.'$#',$file)) continue;
    $data[]  = $dir.'/'.$file;
    $files[] = $file;
  }
  closedir($dh);
  sort($files);
  sort($dirs);

  foreach ($dirs as $dirx){
    $sub = preg_match('#^'.$dirx.'(/|$)#',$rec);
    if($rund) $rund($dirx,basename($dirx),$lvl,1,$sub);
    if($sub) $data = array_merge($data,recursedir($base,$rec,$runf,$rund,$ext,$dirx,$lvl+1));
    if($rund) $rund($dirx,basename($dirx),$lvl,0,$sub);
  }
  if($runf){
    foreach ($files as $file){
      $runf($dir,$file,$lvl);
    }
  }
  return $data;

*/

/**
 * downloads a file from the net and saves it to the given location
 */
function download($url,$file){
  $fp = @fopen($url,"rb");
  if(!$fp) return false;

  while(!feof($fp)){
    $cont.= fread($fp,1024);
  }
  fclose($fp);

  $fp2 = @fopen($file,"w");
  if(!$fp2) return false;
  fwrite($fp2,$cont);
  fclose($fp2);
  return true;
} 

/**
 * extracts the query from a google referer
 */
function getGoogleQuery(){
  $url = parse_url($_SERVER['HTTP_REFERER']);

  if(!preg_match("#google\.#i",$url['host'])) return '';
  $query = array();
  parse_str($url['query'],$query);

  return $query['q'];
}

?>
